from collections import namedtuple

Tags = namedtuple('Tag', 'id, tags')
News = namedtuple('News', 'id, tags')
MLData = namedtuple('MLData', 'news_id, tags_id')


class MLDataMaker:
    def __init__(self, db):
        self.db = db
        # TODO Probably use dict for it
        self.tags = [Tags(id=i[0], tags=self.split_tags(i[1], '/() ')) for i in self.db.select('select id, name from wp_esi_tag')]
        self.alias = self._make_alias_dict(self.db.select('select tag_id, alias from wp_esi_tag_alias'))

    def split_tags(self, tags, char, replace=' '):
        if len(char) > 1:
            for i in char:
                tags = tags.replace(i, replace)
            tags = ' '.join([i for i in tags.split(replace) if len(i) > 0]).lower()
            return tags.split(replace)
        return tags.split(char)

    def is_tag_similar(self, tag, news_tag):
        count = 0
        for i in news_tag.split():
            if i.lower() in tag:
                count += 1
        if count == 0 or count < (len(tag) // 2 + 1):
            return False
        else:
            return True

    def make_ml_data(self, news):
        ml_data = list()
        if not news.tags or news.tags == []:
            tags = ''
        else:
            tags = news.tags
        for news_tag in tags.split(','):
            for i in self.tags:
                if self.is_tag_similar(i.tags, news_tag):
                    ml_data.append(MLData(news.id, i.id))
            for k, v in self.alias.items():
                if self.is_tag_similar(v, news_tag):
                    ml_data.append(MLData(news.id, k))
        if len(ml_data) > 0:
            return ml_data
        else:
            return None

    def _make_alias_dict(self, alias):
        result = dict()
        for i in alias:
            if i[0] in result:
                result[i[0]].append(i[1])
            else:
                result[i[0]] = [i[1]]
        return result
